// Common.h : header file
//

#include <stdexcept>

#pragma once

namespace Common
{
   /// \brief Returns latest COM error description
   /// \param spClass [in,out] - Pointer to COM object which raised error
   /// \param descr [out] - Error description
   /// \return Result code
   template <class IFaceT>
      STDMETHODIMP GetErrorString(const ATL::CComPtr<IFaceT>& spClass, ATL::CComBSTR& descr)
   {
      HRESULT hr = S_OK;

      descr.Empty();

      ATL::CComPtr<ISupportErrorInfo> spSupportErrorInfo;
      hr = spClass->QueryInterface(__uuidof(ISupportErrorInfo), (void** )&spSupportErrorInfo);

      if (SUCCEEDED(hr))
      {
         hr = spSupportErrorInfo->InterfaceSupportsErrorInfo(__uuidof(IFaceT));
         if (hr == S_OK)
         {
            ATL::CComPtr<IErrorInfo> spErrorInfo;
            hr = GetErrorInfo(0, &spErrorInfo);

            if (SUCCEEDED(hr))
            {
               hr = spErrorInfo->GetDescription(&descr);
            }
         }
      }
      return hr;
   }

   /// \brief Throws runtime exception depending on passed result code.
   /// \param spIFace - Pointer to object which is the source of the result code
   /// \param hr - Result code
   template <class IFaceT>
      void ThrowOnCOMError(
      const ATL::CComPtr<IFaceT>& spIFace, 
      HRESULT hr)
   {
      if (FAILED(hr)) 
      {
         HRESULT hResult;
         ATL::CComBSTR errMessage;

         ATL::CComBSTR errDescCOM;
         hResult = GetErrorString(spIFace, errDescCOM);
         if (hResult == S_OK)
         {
            errMessage.Append(L"COM error occurred. Description: ");
            errMessage.Append(errDescCOM);
         }
         else
         {
            char* errDesc = NULL;
            FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
               NULL,
               hr,
               MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
               (LPTSTR)&errDesc,
               0,
               NULL);
            errMessage.Append(errDesc);
            LocalFree(errDesc);
         }
         throw std::runtime_error(std::string(ATL::CW2A(errMessage.m_str)));
      }
   }
}

/// \brief Converts raw date format to string
/// \param tsTime - Raw date
/// \param withTime - Add time to string
/// \param withDate - Add date to string
/// \return withDate - Formated string of date and time
CString DateTimeToString(DATE tsDateTime, 
                        bool withDate = true, 
                        bool withTime = true,
                        bool withSeconds = false);

/// \brief Converts variant value to string
/// \param value - Variant value
/// \return withDate - String representation of the value
CString VariantToString(const ATL::CComVariant& value);

/// \brief Insert objects to the list
/// \param gwAccountID - GW account id
/// \param nameCollection - objects name collection
/// \param trView - View to show objects names
/// \param spCQGCEL - CQGCEL instance
template <typename NameCollection>
void InsertTradableObjectTreeNode(long gwAccountID,
                                  NameCollection* nameCollection,
                                  CTreeCtrl& trView,
                                  ATL::CComPtr<ICQGCEL> spCQGCEL)
{
   try
   {
      HRESULT hr;

      // Add account as node to tree
      HTREEITEM hTrItem;
      if (gwAccountID == 0)
      {
         hTrItem = trView.InsertItem(_T("All Accounts"));
      }
      else
      {
         ATL::CComPtr<ICQGAccounts> spAccounts;
         hr = spCQGCEL->get_Accounts(&spAccounts);
         Common::ThrowOnCOMError(spCQGCEL, hr);

         ATL::CComPtr<ICQGAccount> spAccount;
         hr = spAccounts->get_Item(gwAccountID, &spAccount);
         Common::ThrowOnCOMError(spAccounts, hr);

         ATL::CComBSTR accNameGW;
         hr = spAccount->get_GWAccountName(&accNameGW);
         Common::ThrowOnCOMError(spAccount, hr);

         hTrItem = trView.InsertItem(ATL::CW2T(accNameGW));
      }

      // Insert all tradable objects as sub nodes to added account node 
      long count;
      hr = nameCollection->get_Count(&count);
      Common::ThrowOnCOMError<NameCollection>(nameCollection, hr);

      for (int i = 0; i < count; ++i)
      {
         ATL::CComBSTR objectName;
         hr = nameCollection->get_Item(i, &objectName);
         Common::ThrowOnCOMError<NameCollection>(nameCollection, hr);

         trView.InsertItem(ATL::CW2T(objectName), hTrItem);
      }

      trView.SortChildren(hTrItem);
      trView.SortChildren(TVI_ROOT);
   }
   catch (std::runtime_error& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }
}